package com.css.simulation.resource.scheduler.web.scheduler;

import api.common.pojo.constants.DictConstants;
import api.common.util.CollectionUtil;
import com.css.simulation.resource.scheduler.common.configuration.kubernetes.KubernetesConfiguration;
import com.css.simulation.resource.scheduler.dao.mapper.AutoSubProjectMapper;
import com.css.simulation.resource.scheduler.dao.mapper.ManualProjectMapper;
import com.css.simulation.resource.scheduler.dao.entity.ProjectPO;
import com.css.simulation.resource.scheduler.service.domain.KubernetesNodeTO;
import com.css.simulation.resource.scheduler.common.util.KubernetesUtil;
import com.css.simulation.resource.scheduler.common.util.RedisUtil;
import io.kubernetes.client.openapi.ApiClient;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.List;

@Component
@Slf4j
public class ClusterScheduler {


    @Resource
    private KubernetesConfiguration kubernetesConfiguration;
    @Resource
    private ApiClient apiClient;
    @Resource
    private  StringRedisTemplate stringRedisTemplate;
    @Resource
    private ManualProjectMapper manualProjectMapper;
    @Resource
    private AutoSubProjectMapper autoSubProjectMapper;

    /**
     * 闲时重置所有并行度
     */
    @Scheduled(fixedDelay = 2 * 60 * 1000)
    @SneakyThrows
    public void initNodeParallelism() {
        List<KubernetesNodeTO> initialNodeList = kubernetesConfiguration.getNodeList(); // 预设并行度的节点列表
        List<String> podNameList = KubernetesUtil.getPod(apiClient, kubernetesConfiguration.getNamespace());
        if (CollectionUtil.isEmpty(podNameList)) {
            for (KubernetesNodeTO kubernetesNodeTO : initialNodeList) {
                stringRedisTemplate.opsForValue().set("node:" + kubernetesNodeTO.getName() + ":parallelism", kubernetesNodeTO.getMaxParallelism() + "");
                log.info("ClusterScheduler--initNodeParallelism 重置节点 " + kubernetesNodeTO.getName() + " 并行度为：" + kubernetesNodeTO.getMaxParallelism());
            }
        }
    }

    /**
     * 闲时删除 cluster 所有遗留数据
     */
    @Scheduled(fixedDelay = 2 * 60 * 1000)
    @SneakyThrows
    public void removeCluster() {
        //1 查询是否有正在运行的项目
        List<ProjectPO> manual = manualProjectMapper.selectByNowRunState(DictConstants.PROJECT_RUNNING);
        List<ProjectPO> autoSub = autoSubProjectMapper.selectByNowRunState(DictConstants.PROJECT_RUNNING);

        //2 如果没有正在运行的项目，则删除redis
        if (CollectionUtil.isEmpty(manual) && CollectionUtil.isEmpty(autoSub)) {
            RedisUtil.deleteByPrefix(stringRedisTemplate, "cluster:");
            log.info("ClusterScheduler--removeCluster 清除 redis 中因错误残留的 cluster 数据。");
        }
    }
}
